import logging as log

def check_xnat_item(x, a):

    projects = [e.label() for e in list(x.select.projects())]
    experiments = []
    for p in projects:
        exp = x.array.experiments(project_id=p).data
        experiments.extend([e['ID'] for e in exp])

    if a in projects:
        log.info('Project detected: %s'%a)
        return 0
    elif a in experiments:
        log.info('Experiment detected: %s'%a)
        return 1
    else:
        from bx import lists
        if hasattr(lists, a):
            print('List detected: %s'%a)
            return 2
        else:
            log.error('%s is not a project/experiment/list'%a)
            return -1

def collect_experiments(x, id, columns=['label', 'subject_label'], max_rows=1):
    import os
    t = check_xnat_item(x, id)

    if not os.environ.get('CI_TEST', None):
        max_rows = None

    experiments = []
    if t == 0:
        print('t0')
        experiments = x.array.experiments(project_id=id,
                columns=columns).data[:max_rows]
    elif t == 1:
        experiments = [x.array.experiments(experiment_id=id,
                columns=columns).data[0]]
    elif t == 2:
        from bx import lists
        expes = getattr(lists, id)
        for e in expes[:max_rows]:
            ex = x.array.experiments(experiment_id=e,
                    columns=columns).data[0]
            experiments.append(ex)
    else:
        log.error('%s is not a project or an experiment nor a list'%id)

    return experiments



import os
import os.path as op
import logging as log
from tqdm import tqdm


def download_experiment(x, e, resource_name, validator, overwrite, destdir):

    e_id = e['ID']
    e = x.select.experiment(e_id)
    # resource_name is None when downloading reports only
    if not resource_name is None:
        r = e.resource(resource_name)
        if not r.exists():
            log.error('%s has no %s resource'%(e_id, resource_name))
            return
        dd = op.join(destdir, e_id)
        if op.isdir(dd) and not overwrite:
            msg = '%s already exists. Skipping %s.'%(dd, e_id)
            log.error(msg)
        else:
            if op.isdir(dd) and overwrite:
                msg = '%s already exists. Overwriting %s.'%(dd, e_id)
                log.warning(msg)
            elif not op.isdir(dd):
                os.mkdir(dd)
            r.get(dest_dir=dd)

    v = e.resource('BBRC_VALIDATOR')
    f = v.pdf(validator)

    if not f is None:
        fp = op.join(destdir, f.label()) if resource_name is None \
            else op.join(dd, f.label())
        #if resource_name is None:
        log.debug('Saving it in %s.'%fp)
        f.get(dest=fp)
    return destdir


def download(x, experiments, resource_name, validator, destdir, subcommand,
        overwrite=False):

    if subcommand in ['files', 'report']:
        type = 'experiment'
    elif subcommand == 'snapshot':
        type = 'snapshot'
    elif subcommand == 'rc':
        type = 'rc'
    else:
        log.error('Invalid subcommand (%s).'%subcommand)

    if len(experiments) > 1:
        log.info('Now initiating download for %s experiments.'%len(experiments))
        experiments = tqdm(experiments)
    for e in experiments:
        log.debug(e)
        try:
            if type == 'experiment':
                fp = download_experiment(x, e, resource_name, validator, overwrite,
                    destdir)

            elif type == 'snapshot':
                fp = download_snapshot(x, e, resource_name, validator, overwrite,
                        destdir)

            elif type == 'rc':
                fp = download_rc(x, e, resource_name, validator, destdir)
        except KeyboardInterrupt:
            return

    if len(experiments) == 1:
        log.info('Saving it in %s'%fp)

def download_snapshot(x, e, resource_name, validator, overwrite, destdir):
    rn = 'BBRC_VALIDATOR'
    e_id = e['ID']
    r = x.select.experiment(e_id).resource(rn)
    if r.exists():
        fp = op.join(destdir, '%s.jpg'%e_id)
        try :
            r.download_snapshot(validator, fp)
        except Exception as exc:
            log.error('%s has no %s (%s)'%(e_id, validator, exc))
        return fp
    else:
        log.error('%s has no %s'%(e_id, validator))


def download_rc(x, e, resource_name, validator, destdir):
    e_id = e['ID']
    log.debug(e_id)
    subject_label = e['subject_label']

    e = x.select.experiment(e_id)
    r = e.resource(resource_name)
    if not r.exists():
        log.error('%s has no %s resource'%(e_id, resource_name))
        return
    try:
        r.download_rc(destdir)
        v = e.resource('BBRC_VALIDATOR')
        if v.exists():
            fp = op.join(destdir, '%s_%s.jpg'%(subject_label, e_id))
            try:
                v.download_snapshot(validator, fp)
            except Exception as exc:
                log.error('%s has no %s (%s)'%(e_id, validator, exc))
        else:
            log.warning('%s has not %s'%(e, validator))

    except Exception as exc:
        log.error('%s failed. Skipping (%s).'%(e_id, exc))
    return destdir

def measurements(x, experiments, subfunc, resource_name='FREESURFER6'):
    from tqdm import tqdm
    import pandas as pd

    table = []
    for e in tqdm(experiments):
        log.debug(e)
        try:
            s = e['subject_label']
            e_id = e['ID']
            r = x.select.experiment(e_id).resource(resource_name)
            if not r.exists():
                log.error('%s has no %s resource'%(e_id, resource_name))
                continue

            if subfunc == 'aparc':
                volumes = r.aparc()
            elif subfunc == 'aseg':
                volumes = r.aseg()
            elif subfunc == 'hippoSfVolumes':
                volumes = r.hippoSfVolumes(mode='T1')
            elif subfunc == 'volumes':
                volumes = r.volumes()
                if isinstance(volumes, dict):
                    volumes = pd.DataFrame(volumes, index=[s])

            volumes['subject'] = s
            volumes['ID'] = e['ID']
            table.append(volumes)
        except KeyboardInterrupt:
            return pd.concat(table).set_index('ID').sort_index()
        except Exception as exc:
            log.error('Failed for %s. Skipping it. (%s)'%(e, exc))

    data = pd.concat(table).set_index('ID').sort_index()
    return data



import logging as log
import os.path as op
import os
from bx.command import Command

class NiftiCommand(Command):
    '''Download NIfTI images from a given sequence (`SeriesDesc`).

    Usage: bx nifti <subcommand> <resource_id>
    Available subcommands:
     usable:\t\tdownload `usable` images (default)
     all:\t\tdownload all images found
    User is then asked for sequence name (ex: T1, T2, DWI). Has to match with
    the one in XNAT.'''

    nargs = 2
    subcommands = ['usable', 'all']

    def __init__(self, *args, **kwargs):
        super(NiftiCommand, self).__init__(*args, **kwargs)


    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]
        self.sequence_name = 'T1_ALFA1'
        if not os.environ.get('CI_TEST', None):
            s = input('Enter Sequence Name (i.e. Series Description):')
            self.sequence_name = s
        log.info(self.sequence_name)
        self.run_id(id, download_sequence, sequence_name=self.sequence_name,
                    destdir=self.destdir, subcommand=subcommand)


def download_sequence(x, experiments, sequence_name, destdir, subcommand):

    import shutil
    from glob import glob
    import tempfile
    from tqdm import tqdm

    nii_extensions = ['nii.gz','bvec','bval']

    for e in tqdm(experiments):
        log.debug('Experiment %s:'%e['ID'])

        scans = x.select.experiment(e['ID']).scans()
        seq_scans = [s for s in scans if not s.label().startswith('0-') and\
            sequence_name == s.attrs.get('type')]
        if subcommand == 'usable' :
            seq_scans = [s for s in seq_scans if s.attrs.get('quality') == 'usable']
        log.debug('Found following scans: %s'%seq_scans)

        for s in seq_scans:
            r = s.resource('NIFTI')
            if not r.exists():
                log.error('%s has no NIFTI'%e)
                continue
            wd = tempfile.mkdtemp()

            r.get(dest_dir=wd, extract=True)
            for ext in nii_extensions :
                fp = glob(op.join(wd, 'NIFTI', '*.%s' % ext))
                if len(fp) != 1:
                    if ext == 'nii.gz':
                        log.error('Multiple files found in archive. Skipping %s (%s) folder: %s'\
                            %(e['ID'], s.label(), op.join(wd, 'NIFTI')))
                    continue
                fp = fp[0]
                fn = '%s_%s_%s_%s.%s'%(e['subject_label'], e['label'],
                    s.label(), e['ID'], ext)
                log.debug('Saving %s...'%fn)
                shutil.move(fp, op.join(destdir, fn))
            shutil.rmtree(wd)



from bx.command import Command

class IDCommand(Command):
    '''Returns subject/session labels along with parent project.

    Usage: bx id <resource_id>'''
    nargs = 1

    def __init__(self, *args, **kwargs):
        super(IDCommand, self).__init__(*args, **kwargs)

    def parse(self):
        id = self.args[0]
        df = self.run_id(id, get_id_table, max_rows=10)

        if len(df) == 1:
            print(df.iloc[0])
        else:
            self.to_excel(id, df)


def get_id_table(x, experiments):
    import pandas as pd
    table = []
    from tqdm import tqdm
    columns = ['label', 'subject_label']
    for e in tqdm(experiments):
        try:
            e_id = e['ID']
            exp = x.array.experiments(experiment_id=e_id, columns=columns).data[0]
            table.append([e_id, exp['label'], exp['subject_label'], exp['project']])
        except KeyboardInterrupt:
            break

    columns = ['ID', 'label', 'subject_label', 'project']
    df = pd.DataFrame(table, columns=columns).set_index('ID').sort_index()
    return df



import os.path as op
import os
import tempfile
import argparse
import pkgutil
import inspect
import logging as log

def __get_modules__(m):
    modules = []
    prefix = m.__name__ + '.'
    log.info('prefix : %s'%prefix)
    for importer, modname, ispkg in pkgutil.iter_modules(m.__path__, prefix):
        module = __import__(modname , fromlist='dummy')
        if not ispkg:
            modules.append(module)
        else:
            modules.extend(__get_modules__(module))
    return modules

def __find_all_commands__(m):
    ''' Browses bx and looks for any class named as a Command'''
    modules = []
    classes = []
    modules = __get_modules__(m)
    forbidden_classes = [] #Test, ScanTest, ExperimentTest]
    for m in modules:
        for name, obj in inspect.getmembers(m):
            if inspect.isclass(obj) and 'Command' in name \
                    and not obj in forbidden_classes:
                classes.append(obj)
    return classes

class readable_dir(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        prospective_dir=values
        if not os.path.isdir(prospective_dir):
            msg = "readable_dir:{0} is not a valid path".format(prospective_dir)
            raise argparse.ArgumentTypeError(msg)
        if os.access(prospective_dir, os.R_OK):
            setattr(namespace,self.dest,prospective_dir)
        else:
            msg = "readable_dir:{0} is not a readable dir".format(prospective_dir)
            raise argparse.ArgumentTypeError(msg)


def parse(parser, command):
    c = command(parser.command, parser.args, parser.xnat, parser.destdir,
        parser.overwrite)

    if len(c.args) == 0:
        msg = c.__doc__
        cn = parser.command
        print('\nHelp for command `%s`:\n\n%s'%(cn, msg))

    elif len(c.args) == command.nargs:
        if command.nargs == 1 or c.args[0] in command.subcommands:
            c.parse()
        else:
            subcommands = '\n - '.join(command.subcommands)
            msg = '\n%s invalid\n\nAvailable subcommands:\n - %s'%(c.args[0], subcommands)
            print(msg)

    else:
        msg = c.__doc__
        cn = parser.command
        msg = '\n\nMissing argument(s)\n\nHelp for command `%s`:\n\n%s'%(cn, msg)
        print(msg)


def parse_args(command, args, x, destdir=tempfile.gettempdir(), overwrite=False):
    from bx.command import Command
    parser = Command(command, args, x, destdir, overwrite=overwrite)

    import bx
    commands = __find_all_commands__(bx)
    commands = {e.__name__.split('.')[-1].lower()[:-7]: e for e in commands}

    if command in commands.keys():

        log.debug('Command: %s'%command)
        if command == 'nifti' and len(args) == 1:
            args.insert(0, 'usable')
        if command in ['freesurfer6']:
            ans = ''

            if not os.environ.get('CI_TEST', None):
                while not ans in ['1', '2']:
                    msg = 'Please confirm if you want FREESURFER6 or FREESURFER6_HIRES.'\
                      '(1) FREESURFER6  (2) FREESURFER6_HIRES ?'
                    ans = input(msg)

            if ans == '2':
                command = 'freesurfer6hires'
                parser = Command(command, args, x, destdir, overwrite=overwrite)

        parse(parser, commands[command])

    else:
        msg = '%s not found \n\nValid commands:\n %s'\
            %(command, '\n '.join([e for e, v in commands.items() if e!='']))
        log.error(msg)
        #raise Exception(msg)



def create_parser():
    import argparse
    cfgfile = op.join(op.expanduser('~'), '.xnat.cfg')

    import bx
    commands = __find_all_commands__(bx)
    commands = {e.__name__.split('.')[-1].lower()[:-7]: e.__doc__ \
        for e in commands if e.__name__ != 'Command'}
    from bx import __version__
    epilog = 'bx (v%s)\n\nExisting commands:\n'%__version__

    for e, v in commands.items():
        i = int(len(str(e)) / 6)
        tabs = (3 - i) * '\t'
        v = '%s%s'%(tabs, v) if not v is None else ''
        epilog = epilog + ' %s %s\n'%(e, str(v).split('\n')[0])

    epilog = epilog + '\nbx is distributed in the hope that it will be useful, '\
     'but WITHOUT ANY WARRANTY. \nSubmit issues/comments/PR at http://gitlab.com/xgrg/bx.\n\n'\
     'Authors: Greg Operto, Jordi Huguet - BarcelonaBeta Brain Research Center (2019)'

    from argparse import RawTextHelpFormatter
    parser = argparse.ArgumentParser(description=epilog,
        formatter_class=RawTextHelpFormatter) #, epilog=epilog)
    parser.add_argument('command', help='bx command')
    parser.add_argument('args', help='bx command', nargs="*")
    parser.add_argument('--config', help='XNAT configuration file',
        required=False, default=cfgfile)
    parser.add_argument('--dest', help='Destination folder',
        required=False, action=readable_dir)
    parser.add_argument('--verbose', '-v', action='store_true', default=False,
        help='Display verbosal information (optional)', required=False)
    parser.add_argument('--overwrite', '-O', action='store_true', default=False,
        help='Overwrite', required=False)

    from bx import __version__
    parser.add_argument('-V', '--version', action='version', version="%(prog)s ("+__version__+")")

    return parser



from datetime import datetime
import os.path as op
import logging as log
import os

class Command(object):
    def __init__(self, command, args, xnat_instance, destdir, overwrite):
        self.command = command
        self.args = args
        self.xnat = xnat_instance
        self.destdir = destdir
        self.overwrite = overwrite

    def run_id(self, id, func, **kwargs):
        from bx import xnat

        is_max_rows = False
        if 'max_rows' in kwargs:
            max_rows = kwargs['max_rows']
            kwargs.pop('max_rows')
            is_max_rows = True
        if not 'CI_TEST' in os.environ.keys():
            max_rows = None
            is_max_rows = True
        if is_max_rows:
            experiments = xnat.collect_experiments(self.xnat, id, max_rows=max_rows)
        else:
            experiments = xnat.collect_experiments(self.xnat, id)
        return func(self.xnat, experiments, **kwargs)

    def to_excel(self, id, df):
        dt = datetime.today().strftime('%Y%m%d_%H%M%S')
        fn = 'bx_%s_%s_%s.xlsx'%(self.command, id, dt)
        fp = op.join(self.destdir, fn)
        log.info('Saving it in %s'%fp)
        df.to_excel(fp)



from bx.command import Command
import tempfile

class MRDatesCommand(Command):
    '''Collect acquisition dates from MR sessions.

    Usage: bx mrdates <resource_id>'''

    nargs = 1

    def __init__(self, *args, **kwargs):
        super(MRDatesCommand, self).__init__(*args, **kwargs)

    def parse(self):
        id = self.args[0] #should be a project or an experiment_id
        print(id)
        from bx.dicom import collect_mrdates
        res = self.run_id(id, collect_mrdates)
        if len(res) == 1:
            print(res.iloc[0])
        else:
            if self.destdir == None:
                self.destdir = tempfile.gettempdir()
            self.to_excel(id, res)



from bx.command import Command
from bx import download as dl

class ASHSCommand(Command):
    '''ASHS (Hippocampal subfield segmentation)

    Available subcommands:
     files:\t\tdownload all `ASHS` outputs (segmentation maps, volumes, everything...)
     volumes:\t\tcreates an Excel table with all hippocampal subfield volumes
     snapshot:\t\tdownload a snapshot from the `ASHS` pipeline
     report:\t\tdownload the validation report issued by `ASHSValidator`
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `ASHSValidator`
    '''
    nargs = 2
    resource_name = 'ASHS'
    subcommands = ['volumes', 'files', 'report', 'snapshot', 'tests']
    validator = 'ASHSValidator'

    def __init__(self, *args, **kwargs):
        super(ASHSCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1] #should be a project or an experiment_id
        if subcommand in ['volumes']:
            df = self.run_id(id, dl.measurements, subfunc=subcommand,
                resource_name=self.resource_name)
            self.to_excel(id, df)


        elif subcommand in ['files', 'report', 'snapshot']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                    validator=self.validator, destdir=self.destdir,
                    overwrite=self.overwrite, subcommand=subcommand)

        elif subcommand == 'tests':
            version=['beed8758', 'f80f2b13']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores, validator=self.validator,
                version=version, max_rows=25)
            self.to_excel(id, df)



from bx.command import Command

class PetCommand(Command):
    '''Pet - used to collect automatic tests from `PetSessionValidator`

    Available subcommands:
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `PetSessionValidator`
    '''
    nargs = 2
    resource_name = 'PetSessionValidator'
    subcommands = ['tests']

    def __init__(self, *args, **kwargs):
        super(PetCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]
        if subcommand == 'tests':
            version=['*', '0390c55f']

            from bx import validation as val
            df = self.run_id(id, val.validation_scores, validator=self.resource_name,
                version=version, max_rows=25)
            self.to_excel(id, df)



from bx.command import Command
from bx import download as dl


class FreeSurfer6Command(Command):
    '''FreeSurfer v6.0

    Available subcommands:
     files:\t\tdownload all `recon-all` outputs (segmentation maps, files, everything...)
     aseg:\t\tcreates an Excel table with all `aseg` measurements
     aparc:\t\tcreates an Excel table with all `aparc` measurements
     hippoSfVolumes:\tsaves an Excel table with hippocampal subfield volumes (if available)
     snapshot:\t\tdownload a snapshot from the `recon-all` pipeline
     report:\t\tdownload the validation report issued by `FreeSurferValidator`
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `FreeSurferValidator`
    '''
    nargs = 2
    resource_name = 'FREESURFER6'
    subcommands = ['aparc', 'aseg', 'hippoSfVolumes', 'snapshot', 'tests', 'report', 'files']
    validator = 'FreeSurferValidator'

    def __init__(self, *args, **kwargs):
        super(FreeSurfer6Command, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1] #should be a project or an experiment_id

        if subcommand in ['aparc', 'aseg', 'hippoSfVolumes']:

            df = self.run_id(id, dl.measurements,
                    resource_name=self.resource_name, subfunc=subcommand,
                    max_rows=3)
            self.to_excel(id, df)

        elif subcommand in ['files', 'report', 'snapshot']:
            self.subcommand_download()

        elif subcommand == 'tests':

            version = ['##0390c55f', '4e37c9d0']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores, validator=self.validator,
                version=version, max_rows=25)
            self.to_excel(id, df)

    def subcommand_download(self):
        subcommand = self.args[0]
        id = self.args[1]

        resource_name = self.resource_name
        validator = self.validator
        if 'hires' in self.command:
            resource_name = 'FREESURFER6_HIRES'
            validator = 'FreeSurferHiresValidator'

        if subcommand == 'report':
            resource_name = None

        from bx import download as dl
        self.run_id(id, dl.download, resource_name=resource_name,
                validator=validator, destdir=self.destdir,
                overwrite=self.overwrite, subcommand=subcommand)


class FreeSurfer6HiresCommand(FreeSurfer6Command):
    '''FreeSurfer v6.0 (-hires option)

    Available subcommands:
     files:\t\tdownload all `recon-all` outputs (segmentation maps, files, everything...)
     aseg:\t\tcreates an Excel table with all `aseg` measurements
     aparc:\t\tcreates an Excel table with all `aparc` measurements
     hippoSfVolumes:\tsaves an Excel table with hippocampal subfield volumes (if available)
     snapshot:\t\tdownload a snapshot from the `recon-all` pipeline
     report:\t\tdownload the validation report issued by `FreeSurferHiresValidator`
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `FreeSurferValidator`
    '''
    nargs = 2
    resource_name = 'FREESURFER6_HIRES'


    def __init__(self, *args, **kwargs):
        super(FreeSurfer6HiresCommand, self).__init__(*args, **kwargs)



from bx.command import Command
from bx import download as dl


class DTIFITCommand(Command):
    '''Processing of Diffusion-weighted Imaging data

    Available subcommands:
    files:\t\tdownload all outputs from the `DTIFIT` pipeline (up to parametric maps)
    report:\t\tdownload the validation report issued by `DTIFITValidator`
    snapshot:\t\tdownload a snapshot with FA map, RGB tensor and TOPUP distortion correction map
    '''
    nargs = 2
    resource_name = 'TOPUP_DTIFIT'
    subcommands = ['files', 'report', 'snapshot', 'tests']
    validator = 'DTIFITValidator'

    def __init__(self, *args, **kwargs):
        super(DTIFITCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1] #should be a project or an experiment_id

        if subcommand in ['files', 'report', 'snapshot']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                    validator=self.validator, destdir=self.destdir,
                    overwrite=self.overwrite, subcommand=subcommand)

        elif subcommand == 'tests':
            version = ['*', '4e37c9d0']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores, validator=self.validator,
                version=version, max_rows=25)
            self.to_excel(id, df)



from bx.command import Command


class QMENTACommand(Command):
    nargs = 2

    def __init__(self, *args, **kwargs):
        super(QMENTACommand, self).__init__(*args, **kwargs)

    def parse(self):
        args = self.args
        subcommand = args[0]
        id = args[1]

        if subcommand == 'files':

            resource_name = 'QMENTA_RESULTS'
            validation_report = 'QMENTA'

            from bx import parse
            parse.download_experiments(self.xnat, id, resource_name,
                validation_report, self.destdir, self.overwrite)



from bx.command import Command
from bx import download as dl
import logging as log

class SignatureCommand(Command):
    '''Download composite measurements labeled as 'signatures' of Alzheimer's Disease

    Available subcommands:
     thickness:\t\tbased on FreeSurfer's cortical thickness
     grayvol:\t\tbased on FreeSurfer's local cortical gray matter volumes

    Signatures are calculated in two versions, weighted and not.
    Weighted means that the formula has been applied by normalizing each
    ROI value by local surface area (as explained in the papers).
    Not-weighted versions correspond to mean values across regions.

    References:
    - Jack et al., Alzheimers Dement. 2017
    - Dickerson et al., Neurology, 2011

    '''
    nargs = 2
    subcommands = ['thickness', 'grayvol']


    def __init__(self, *args, **kwargs):
        super(SignatureCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1] #should be a project or an experiment_id

        if subcommand in ['thickness', 'grayvol']:
            from bx import xnat
            experiments = xnat.collect_experiments(self.xnat, id, max_rows=10)
            d = {'thickness': 'ThickAvg', 'grayvol': 'GrayVol'}
            df = signatures(self.xnat, experiments, d[subcommand])
            self.to_excel(id, df)


def __signature__(x, experiment_id, regions, weighted=True, measurement='ThickAvg'):
    import pyxnat
    import pandas as pd
    e = x.select.experiment(experiment_id)
    resource_name = 'FREESURFER6'

    r = e.resource(resource_name)
    aparc = r.aparc()

    weighted_sum = 0
    total_surf_area = 0

    query = 'region == "{region}" & side == "{side}" & \
             measurement == "{measurement}"'

    for r in regions:
        for s in ['left', 'right']:
            q = query.format(region=r, side=s, measurement=measurement)
            thickness = float(aparc.query(q)['value'])

            q = query.format(region=r, side=s, measurement='SurfArea')
            surf_area = int(aparc.query(q)['value'])

            weighted_sum += thickness * surf_area if weighted else thickness
            total_surf_area += surf_area

    if weighted:
        final = weighted_sum / total_surf_area
    else:
        final = weighted_sum / (2 * len(regions))

    return final


def signature(x, experiment_id, measurement=None):
    import pandas as pd

    columns = ['signature', 'weighted', 'measurement', 'value']
    table = []
    for sig in ['jack', 'dickerson']:

        if sig == 'jack':
            regions = ['entorhinal', 'inferiortemporal', 'middletemporal',
                       'fusiform']
        elif sig == 'dickerson':
            regions = ['middletemporal', 'inferiortemporal', 'temporalpole',
                'inferiorparietal', 'superiorfrontal', 'superiorparietal',
                'supramarginal', 'precuneus', 'parstriangularis',
                'parsopercularis', 'parsorbitalis']

        for weighted in [False, True]:
            meas = ['ThickAvg', 'GrayVol']
            if measurement:
                meas = [measurement]
            for m in meas:
                res = __signature__(x, experiment_id, regions, weighted, m)
                row = [sig, weighted, m, res]
                table.append(row)
    return pd.DataFrame(table, columns=columns)

def signatures(x, experiments, measurement=None):
    from tqdm import tqdm
    import pandas as pd

    table = []
    for e in tqdm(experiments):
        log.debug(e)
        try:
            s = e['subject_label']
            e_id = e['ID']
            volumes = signature(x, e_id, measurement=measurement)

            volumes['subject'] = s
            volumes['ID'] = e['ID']
            table.append(volumes)
        except KeyboardInterrupt:
            return pd.concat(table).set_index('ID').sort_index()
        except Exception as exc:
            log.error('Failed for %s. Skipping it. (%s)'%(e, exc))

    data = pd.concat(table).set_index('ID').sort_index()
    return data



import setuptools
from setuptools import setup

import os.path as op
this_directory = op.abspath(op.dirname(__file__))
with open(op.join(this_directory, 'README.md')) as f:
    long_description = f.read()
from bx import __version__

setup(
  name = 'bbrc-bx',
  install_requires=['coverage>=4.5',
    'nose>=1.3',
    'requests>=2.21',
    'six>=1.10',
    'lxml>=4.3',
    'nibabel>=2.3',
    'pydicom>=1.2',
    'numpy>=1.16',
    'pandas>=0.24',
    'xlrd>=1.2',
    'xlutils>=2.0',
    'xlwt>=1.3',
    'tqdm>=4.31',
    'openpyxl>=2.6',
    'bbrc-pyxnat>=1.2.1.2.post2'],
  scripts=['bin/bx'],
  long_description=long_description,
  long_description_content_type='text/markdown',
  version = __version__,
  description = 'BarcelonaBeta + XNAT = bx',
  packages=setuptools.find_packages(),
  author = 'Greg Operto',
  author_email = 'goperto@barcelonabeta.org',
  url = 'https://gitlab.com/xgrg/bx',
  download_url = 'https://gitlab.com/xgrg/bx/-/archive/v0.0.9.6/bx-v0.0.9.6.tar.gz',
  classifiers = ['Intended Audience :: Science/Research',
      'Intended Audience :: Developers',
      'Topic :: Scientific/Engineering',
      'Operating System :: Unix',
      'Programming Language :: Python :: 3.7' ],
  package_data={'bbrc-bx': [ 'requirements.txt', 'README.md'], },
)



from bx.command import Command
from bx import download as dl

class ANTSCommand(Command):
    '''ANTS

    Available subcommands:
     files:\t\tdownload all `ASHS` outputs (segmentation maps, volumes, everything...)
     snapshot:\t\tdownload a snapshot from the `ASHS` pipeline
     report:\t\tdownload the validation report issued by `ASHSValidator`
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `ASHSValidator`
    '''
    nargs = 2
    resource_name = 'ANTS'
    subcommands = ['files', 'report', 'snapshot', 'tests']
    validator = 'ANTSValidator'

    def __init__(self, *args, **kwargs):
        super(ANTSCommand, self).__init__(*args, **kwargs)


    def parse(self):

        subcommand = self.args[0]
        id = self.args[1] #should be a project or an experiment_id

        if subcommand in ['files', 'report', 'snapshot']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                    validator=self.validator, destdir=self.destdir,
                    overwrite=self.overwrite, subcommand=subcommand)

        elif subcommand == 'tests':
            version=['*', '0390c55f']

            from bx import validation as val
            df = self.run_id(id, val.validation_scores, validator=self.validator,
                version=version, max_rows=25)
            self.to_excel(id, df)
